@@ -447,7 +447,7 @@ class BackupManager(RemoteStatusMixin):
                 "Backup completed (start time: %s, elapsed time: %s)",
                 self.executor.copy_start_time,
                 human_readable_timedelta(
-                    executor.copy_end_time - executor.copy_start_time))
+                    datetime.datetime.now() - executor.copy_start_time))
             # Create a restore point after a backup
             target_name = 'barman_%s' % backup_info.backup_id
             self.server.postgres.create_restore_point(target_name)
@@ -494,6 +494,7 @@ class BackupManager(RemoteStatusMixin):
         :kwparam str|None target_tli: the target timeline
         :kwparam str|None target_time: the target time
         :kwparam str|None target_xid: the target xid
+        :kwparam str|None target_lsn: the target LSN
         :kwparam str|None target_name: the target name created previously with
             pg_create_restore_point() function call
         :kwparam bool|None target_immediate: end recovery as soon as
@@ -511,15 +512,23 @@ class BackupManager(RemoteStatusMixin):
 
         # Run the pre_recovery_script if present.
         script = HookScriptRunner(self, 'recovery_script', 'pre')
-        script.env_from_recover(backup_info, dest, tablespaces, remote_command,
-                                **kwargs)
+        script.env_from_recover(
+            backup_info,
+            dest,
+            tablespaces,
+            remote_command,
+            **kwargs)
         script.run()
 
         # Run the pre_recovery_retry_script if present.
         retry_script = RetryHookScriptRunner(
             self, 'recovery_retry_script', 'pre')
-        script.env_from_recover(backup_info, dest, tablespaces, remote_command,
-                                **kwargs)
+        retry_script.env_from_recover(
+            backup_info,
+            dest,
+            tablespaces,
+            remote_command,
+            **kwargs)
         retry_script.run()
 
         # Execute the recovery.
@@ -535,8 +544,12 @@ class BackupManager(RemoteStatusMixin):
         try:
             retry_script = RetryHookScriptRunner(
                 self, 'recovery_retry_script', 'post')
-            script.env_from_recover(
-                backup_info, dest, tablespaces, remote_command, **kwargs)
+            retry_script.env_from_recover(
+                backup_info,
+                dest,
+                tablespaces,
+                remote_command,
+                **kwargs)
             retry_script.run()
         except AbortedRetryHookScript as e:
             # Ignore the ABORT_STOP as it is a post-hook operation
@@ -1044,6 +1057,11 @@ class BackupManager(RemoteStatusMixin):
         end_wal = backup_info.end_wal
         timeline = begin_wal[:8]
 
+        # Case 0: there is nothing to check for this backup, as it is
+        # currently in progress
+        if not end_wal:
+            return
+
         # Case 1: Barman still doesn't know about the timeline the backup
         # started with. We still haven't archived any WAL corresponding
         # to the backup, so we can't proceed with checking the existence
