@@ -218,6 +218,13 @@ def backup_completer(prefix, parsed_args, **kwargs):
 @arg('--jobs', '-j',
      help='Run the copy in parallel using NJOBS processes.',
      type=check_positive, metavar='NJOBS')
+@arg('--bwlimit',
+     help="maximum transfer rate in kilobytes per second. "
+          "A value of 0 means no limit. Overrides 'bandwidth_limit' "
+          "configuration option.",
+     metavar='KBPS',
+     type=check_non_negative,
+     default=SUPPRESS)
 @expects_obj
 def backup(args):
     """
@@ -241,6 +248,8 @@ def backup(args):
             server.config.immediate_checkpoint = args.immediate_checkpoint
         if args.jobs is not None:
             server.config.parallel_jobs = args.jobs
+        if hasattr(args, 'bwlimit'):
+            server.config.bandwidth_limit = args.bwlimit
         with closing(server):
             server.backup()
     output.close_and_exit()
@@ -354,6 +363,7 @@ def rebuild_xlogdb(args):
      help='target time. You can use any valid unambiguous representation. '
           'e.g: "YYYY-MM-DD HH:MM:SS.mmm"')
 @arg('--target-xid', help='target transaction ID')
+@arg('--target-lsn', help='target LSN (Log Sequence Number)')
 @arg('--target-name',
      help='target name created previously with '
           'pg_create_restore_point() function call')
@@ -362,7 +372,7 @@ def rebuild_xlogdb(args):
      action='store_true',
      default=False)
 @arg('--exclusive',
-     help='set target xid to be non inclusive', action="store_true")
+     help='set target to be non inclusive', action="store_true")
 @arg('--tablespace',
      help='tablespace relocation rule',
      metavar='NAME:LOCATION', action='append')
@@ -378,6 +388,13 @@ def rebuild_xlogdb(args):
      help='specifies the backup ID to recover')
 @arg('destination_directory',
      help='the directory where the new server is created')
+@arg('--bwlimit',
+     help="maximum transfer rate in kilobytes per second. "
+          "A value of 0 means no limit. Overrides 'bandwidth_limit' "
+          "configuration option.",
+     metavar='KBPS',
+     type=check_non_negative,
+     default=SUPPRESS)
 @arg('--retry-times',
      help='Number of retries after an error if base backup copy fails.',
      type=check_non_negative)
@@ -427,7 +444,7 @@ def rebuild_xlogdb(args):
 @expects_obj
 def recover(args):
     """
-    Recover a server at a given time or xid
+    Recover a server at a given time, name, LSN or xid
     """
     server = get_server(args)
 
@@ -485,10 +502,12 @@ def recover(args):
             server.config.recovery_options.remove(RecoveryOptions.GET_WAL)
     if args.jobs is not None:
         server.config.parallel_jobs = args.jobs
+    if hasattr(args, 'bwlimit'):
+        server.config.bandwidth_limit = args.bwlimit
 
     # PostgreSQL supports multiple parameters to specify when the recovery
-    # process will end, and in that case the last entry in recovery.conf
-    # will be used. See [1]
+    # process will end, and in that case the last entry in recovery
+    # configuration files will be used. See [1]
     #
     # Since the meaning of the target options is not dependent on the order
     # of parameters, we decided to make the target options mutually exclusive.
@@ -497,7 +516,7 @@ def recover(args):
     #   recovery-target-settings.html
 
     target_options = ['target_tli', 'target_time', 'target_xid',
-                      'target_name', 'target_immediate']
+                      'target_lsn', 'target_name', 'target_immediate']
     specified_target_options = len(
         [option for option in target_options if getattr(args, option)])
     if specified_target_options > 1:
@@ -523,6 +542,7 @@ def recover(args):
                            target_tli=args.target_tli,
                            target_time=args.target_time,
                            target_xid=args.target_xid,
+                           target_lsn=args.target_lsn,
                            target_name=args.target_name,
                            target_immediate=args.target_immediate,
                            exclusive=args.exclusive,
@@ -687,8 +707,7 @@ def sync_info(args):
         if getattr(args, 'primary', False):
             primary_info = server.primary_node_info(args.last_wal,
                                                     args.last_position)
-            output.info(json.dumps(primary_info, sys.stdout,
-                                   cls=BarmanEncoder, indent=4),
+            output.info(json.dumps(primary_info, cls=BarmanEncoder, indent=4),
                         log=False)
         else:
             server.sync_status(args.last_wal, args.last_position)
@@ -843,6 +862,12 @@ def delete(args):
      metavar='SIZE',
      type=check_positive,
      default=SUPPRESS)
+@arg('--test', '-t',
+     help="test both the connection and the configuration of the requested "
+          "PostgreSQL server in Barman for WAL retrieval. With this option, "
+          "the 'wal_name' mandatory argument is ignored.",
+     action='store_true',
+     default=SUPPRESS)
 @expects_obj
 def get_wal(args):
     """
@@ -852,6 +877,11 @@ def get_wal(args):
     """
     server = get_server(args, inactive_is_error=True)
 
+    if getattr(args, 'test', None):
+        output.info("Ready to retrieve WAL files from the server %s",
+                    server.config.name)
+        return
+
     # Retrieve optional arguments. If an argument is not specified,
     # the namespace doesn't contain it due to SUPPRESS default.
     # In that case we pick 'None' using getattr third argument.
@@ -871,6 +901,12 @@ def get_wal(args):
 @arg('server_name',
      completer=server_completer,
      help='specifies the server name for the command')
+@arg('--test', '-t',
+     help='test both the connection and the configuration of the requested '
+          'PostgreSQL server in Barman to make sure it is ready to receive '
+          'WAL files.',
+     action='store_true',
+     default=SUPPRESS)
 @expects_obj
 def put_wal(args):
     """
@@ -878,6 +914,12 @@ def put_wal(args):
     directory. The file will be read from standard input in tar format.
     """
     server = get_server(args, inactive_is_error=True)
+
+    if getattr(args, 'test', None):
+        output.info("Ready to accept WAL files for the server %s",
+                    server.config.name)
+        return
+
     try:
         # Python 3.x
         stream = sys.stdin.buffer
@@ -1068,7 +1110,7 @@ def get_server(args, skip_inactive=True, skip_disabled=False,
     :param bool inactive_is_error: treat inactive server as error
     :param bool on_error_stop: stop if an error is found
     :param bool suppress_error: suppress display of errors (e.g. diagnose)
-    :rtype: barman.server.Server|None
+    :rtype: Server|None
     """
     # This function must to be called with in a single-server context
     name = args.server_name
