@@ -26,7 +26,7 @@ from abc import ABCMeta
 import psycopg2
 from psycopg2.errorcodes import (DUPLICATE_OBJECT, OBJECT_IN_USE,
                                  UNDEFINED_OBJECT)
-from psycopg2.extensions import STATUS_IN_TRANSACTION
+from psycopg2.extensions import STATUS_IN_TRANSACTION, STATUS_READY
 from psycopg2.extras import DictCursor, NamedTupleCursor
 
 from barman.exceptions import (ConninfoException, PostgresAppNameError,
@@ -172,7 +172,9 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
 
         # Check if the connection works by running 'SELECT 1'
         cursor = None
+        initial_status = None
         try:
+            initial_status = self._conn.status
             cursor = self._conn.cursor()
             cursor.execute(self.CHECK_QUERY)
         except psycopg2.DatabaseError:
@@ -185,6 +187,10 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
             return False
         finally:
             if cursor:
+                # Rollback if initial status was IDLE because the CHECK QUERY
+                # has started a new transaction.
+                if initial_status == STATUS_READY:
+                    self._conn.rollback()
                 cursor.close()
 
         return True
@@ -635,6 +641,54 @@ class PostgreSQLConnection(PostgreSQL):
                           force_str(e).strip())
             return None
 
+    @property
+    def archive_timeout(self):
+        """
+        Retrieve the archive_timeout setting in PostgreSQL
+
+        :return: The archive timeout (in seconds)
+        """
+        try:
+            cur = self._cursor(cursor_factory=DictCursor)
+            # We can't use the `get_setting` method here, because it
+            # uses `SHOW`, returning an human readable value such as "5min",
+            # while we prefer a raw value such as 300.
+            cur.execute("SELECT setting "
+                        "FROM pg_settings "
+                        "WHERE name='archive_timeout'")
+            result = cur.fetchone()
+            archive_timeout = int(result[0])
+
+            return archive_timeout
+        except ValueError as e:
+            _logger.error("Error retrieving archive_timeout: %s",
+                          force_str(e).strip())
+            return None
+
+    @property
+    def checkpoint_timeout(self):
+        """
+        Retrieve the checkpoint_timeout setting in PostgreSQL
+
+        :return: The checkpoint timeout (in seconds)
+        """
+        try:
+            cur = self._cursor(cursor_factory=DictCursor)
+            # We can't use the `get_setting` method here, because it
+            # uses `SHOW`, returning an human readable value such as "5min",
+            # while we prefer a raw value such as 300.
+            cur.execute("SELECT setting "
+                        "FROM pg_settings "
+                        "WHERE name='checkpoint_timeout'")
+            result = cur.fetchone()
+            checkpoint_timeout = int(result[0])
+
+            return checkpoint_timeout
+        except ValueError as e:
+            _logger.error("Error retrieving checkpoint_timeout: %s",
+                          force_str(e).strip())
+            return None
+
     def get_archiver_stats(self):
         """
         This method gathers statistics from pg_stat_archiver.
@@ -705,9 +759,21 @@ class PostgreSQLConnection(PostgreSQL):
             pg_superuser_settings + pg_settings + pg_query_keys,
             None)
         try:
-            # check for wal_level only if the version is >= 9.0
+            # Retrieve wal_level, hot_standby and max_wal_senders
+            # only if version is >= 9.0
             if self.server_version >= 90000:
                 pg_settings.append('wal_level')
+                pg_settings.append('hot_standby')
+                pg_settings.append('max_wal_senders')
+
+            if self.server_version >= 90300:
+                pg_settings.append('data_checksums')
+
+            if self.server_version >= 90400:
+                pg_settings.append('max_replication_slots')
+
+            if self.server_version >= 90500:
+                pg_settings.append('wal_compression')
 
             # retrieves superuser settings
             if self.is_superuser:
@@ -724,6 +790,9 @@ class PostgreSQLConnection(PostgreSQL):
             result['pgespresso_installed'] = self.has_pgespresso
             result['current_xlog'] = self.current_xlog_file_name
             result['current_size'] = self.current_size
+            result['archive_timeout'] = self.archive_timeout
+            result['checkpoint_timeout'] = self.checkpoint_timeout
+            result['xlog_segment_size'] = self.xlog_segment_size
 
             result.update(self.get_configuration_files())
 
@@ -1209,12 +1278,15 @@ class PostgreSQLConnection(PostgreSQL):
                 raise PostgresUnsupportedFeature('9.1')
 
             from_repslot = ""
+            where_clauses = []
             if self.server_version >= 100000:
                 # Current implementation (10+)
                 what = "r.*, rs.slot_name"
                 # Look for replication slot name
                 from_repslot = "LEFT JOIN pg_replication_slots rs " \
                                "ON (r.pid = rs.active_pid) "
+                where_clauses += ["(rs.slot_type IS NULL OR "
+                                  "rs.slot_type = 'physical')"]
             elif self.server_version >= 90500:
                 # PostgreSQL 9.5/9.6
                 what = "pid, " \
@@ -1237,6 +1309,8 @@ class PostgreSQLConnection(PostgreSQL):
                 # Look for replication slot name
                 from_repslot = "LEFT JOIN pg_replication_slots rs " \
                                "ON (r.pid = rs.active_pid) "
+                where_clauses += ["(rs.slot_type IS NULL OR "
+                                  "rs.slot_type = 'physical')"]
             elif self.server_version >= 90400:
                 # PostgreSQL 9.4
                 what = "pid, " \
@@ -1295,12 +1369,15 @@ class PostgreSQLConnection(PostgreSQL):
             # Streaming client
             if client_type == self.STANDBY:
                 # Standby server
-                where = 'WHERE {replay_lsn} IS NOT NULL '.format(
-                    **self.name_map)
+                where_clauses += ['{replay_lsn} IS NOT NULL'.format(
+                    **self.name_map)]
             elif client_type == self.WALSTREAMER:
                 # WAL streamer
-                where = 'WHERE {replay_lsn} IS NULL '.format(
-                    **self.name_map)
+                where_clauses += ['{replay_lsn} IS NULL'.format(
+                    **self.name_map)]
+
+            if where_clauses:
+                where = 'WHERE %s ' % ' AND '.join(where_clauses)
             else:
                 where = ''
 
@@ -1389,7 +1466,9 @@ class PostgreSQLConnection(PostgreSQL):
             if names_end < 0:
                 names_end = len(synchronous_standby_names)
             names_list = synchronous_standby_names[names_start:names_end]
-            return [x.strip() for x in names_list.split(',')]
+            # We can blindly strip double quotes because PostgreSQL enforces
+            # the format of the synchronous_standby_names content
+            return [x.strip().strip('"') for x in names_list.split(',')]
 
     @property
     def name_map(self):
